/* Public Domain Dedication                                                                             */

/* The person or persons who have associated work with this document (the "Dedicator" or "Certifier")   */
/* hereby either (a) certifies that, to the best of his knowledge, the work of authorship identified is */
/* in the public domain of the country from which the work is published, or (b) hereby dedicates        */
/* whatever copyright the dedicators holds in the work of authorship identified below (the "Work") to   */
/* the public domain. A certifier, moreover, dedicates any copyright interest he may have in the        */
/* associated work, and for these purposes, is described as a "dedicator" below.                        */

/* A certifier has taken reasonable steps to verify the copyright status of this work. Certifier        */
/* recognizes that his good faith efforts may not shield him from liability if in fact the work         */
/* certified is not in the public domain.                                                               */

/* Dedicator makes this dedication for the benefit of the public at large and to the detriment of the   */
/* Dedicator's heirs and successors. Dedicator intends this dedication to be an overt act of            */
/* relinquishment in perpetuity of all present and future rights under copyright law, whether vested or */
/* contingent, in the Work. Dedicator understands that such relinquishment of all rights includes the   */
/* relinquishment of all rights to enforce (by lawsuit or otherwise) those copyrights in the Work.      */

/* Dedicator recognizes that, once placed in the public domain, the Work may be freely reproduced,      */
/* distributed, transmitted, used, modified, built upon, or otherwise exploited by anyone for any       */
/* purpose, commercial or non-commercial, and in any way, including by methods that have not yet been   */
/* invented or conceived.                                                                               */

/*******************************************************************************
* FILE NAME: user_routines.h
*
* DESCRIPTION:
*  This is the include file which corresponds to user_routines.c and
*  user_routines_fast.c
*  It contains some aliases and function prototypes used in those files.
*
* USAGE:
*  If you add your own routines to those files, this is a good place to add
*  your custom macros (aliases), type definitions, and function prototypes.
*******************************************************************************/

#ifndef __user_program_h_
#define __user_program_h_


/*******************************************************************************
                            MACRO DECLARATIONS
*******************************************************************************/
/* Add your macros (aliases and constants) here.                              */
/* Do not edit the ones in ifi_aliases.h                                      */
/* Macros are substituted in at compile time and make your code more readable */
/* as well as making it easy to change a constant value in one place, rather  */
/* than at every place it is used in your code.                               */
/*
 EXAMPLE CONSTANTS:
#define MAXIMUM_LOOPS   5
#define THE_ANSWER      42
#define TRUE            1
#define FALSE           0
#define PI_VAL          3.1415

 EXAMPLE ALIASES:
#define LIMIT_SWITCH_1  rc_dig_int1  (Points to another macro in ifi_aliases.h)
#define MAIN_SOLENOID   solenoid1    (Points to another macro in ifi_aliases.h)
*/

/* Used in limit switch routines in user_routines.c */
#define OPEN        1     /* Limit switch is open (input is floating high). */
#define CLOSED      0     /* Limit switch is closed (input connected to ground). */


/*******************************************************************************
                            TYPEDEF DECLARATIONS
*******************************************************************************/
/* EXAMPLE DATA STRUCTURE */
/*
typedef struct
{
  unsigned int  NEW_CAPTURE_DATA:1;
  unsigned int  LAST_IN1:1;
  unsigned int  LAST_IN2:1;
  unsigned int  WHEEL_COUNTER_UP:1;
  unsigned int  :4;
  unsigned int wheel_left_counter;
  unsigned int wheel_right_counter;
} user_struct;
*/


/*******************************************************************************
                           FUNCTION PROTOTYPES
*******************************************************************************/

/* These routines reside in user_routines.c */
void User_Initialization(void);
void Process_Data_From_Master_uP(void);
void Default_Routine(void);

void Pauls_Routine(void);

/* These routines reside in user_routines_fast.c */
void InterruptHandlerLow (void);  /* DO NOT CHANGE! */
void User_Autonomous_Code(void);  /* Only in full-size FRC system. */
void Process_Data_From_Local_IO(void);


#endif
/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
